@@ -149,14 +149,14 @@ static jk_uint64_t scm(jk_uint64_t a, jk_uint64_t b)
 
 /* Return the string representation of the lb lock type */
 /* based on the integer representation */
-const char *jk_lb_get_lock_direct(int lblock, jk_logger_t *l)
+const char *jk_lb_get_lock_direct(int lblock, jk_log_context_t *l)
 {
     return lb_locking_type[lblock];
 }
 
 /* Return the string representation of the lb lock type */
 /* based on the lb worker struct */
-const char *jk_lb_get_lock(lb_worker_t *p, jk_logger_t *l)
+const char *jk_lb_get_lock(lb_worker_t *p, jk_log_context_t *l)
 {
     return lb_locking_type[p->lblock];
 }
@@ -175,14 +175,14 @@ int jk_lb_get_lock_code(const char *v)
 
 /* Return the string representation of the lb method type */
 /* based on the integer representation */
-const char *jk_lb_get_method_direct(int lbmethod, jk_logger_t *l)
+const char *jk_lb_get_method_direct(int lbmethod, jk_log_context_t *l)
 {
     return lb_method_type[lbmethod];
 }
 
 /* Return the string representation of the lb method type */
 /* based on the lb worker struct */
-const char *jk_lb_get_method(lb_worker_t *p, jk_logger_t *l)
+const char *jk_lb_get_method(lb_worker_t *p, jk_log_context_t *l)
 {
     return lb_method_type[p->lbmethod];
 }
@@ -207,14 +207,14 @@ int jk_lb_get_method_code(const char *v)
 
 /* Return the string representation of the balance worker state */
 /* based on the integer representation */
-const char *jk_lb_get_state_direct(int state, jk_logger_t *l)
+const char *jk_lb_get_state_direct(int state, jk_log_context_t *l)
 {
     return lb_state_type[state];
 }
 
 /* Return the string representation of the balance worker state */
 /* based on the sub worker struct */
-const char *jk_lb_get_state(lb_sub_worker_t *p, jk_logger_t *l)
+const char *jk_lb_get_state(lb_sub_worker_t *p, jk_log_context_t *l)
 {
     return lb_state_type[p->s->state];
 }
@@ -243,14 +243,14 @@ int jk_lb_get_state_code(const char *v)
 
 /* Return the string representation of the balance worker activation */
 /* based on the integer representation */
-const char *jk_lb_get_activation_direct(int activation, jk_logger_t *l)
+const char *jk_lb_get_activation_direct(int activation, jk_log_context_t *l)
 {
     return lb_activation_type[activation];
 }
 
 /* Return the string representation of the balance worker activation */
 /* based on the sub worker struct */
-const char *jk_lb_get_activation(lb_sub_worker_t *p, jk_logger_t *l)
+const char *jk_lb_get_activation(lb_sub_worker_t *p, jk_log_context_t *l)
 {
     return lb_activation_type[p->activation];
 }
@@ -269,7 +269,7 @@ int jk_lb_get_activation_code(const char *v)
 }
 
 /* Update the load multipliers wrt. lb_factor */
-void update_mult(lb_worker_t *p, jk_logger_t *l)
+void update_mult(lb_worker_t *p, jk_log_context_t *l)
 {
     unsigned int i = 0;
     jk_uint64_t s = 1;
@@ -291,7 +291,7 @@ void update_mult(lb_worker_t *p, jk_logger_t *l)
 
 /* Reset all lb values.
  */
-void reset_lb_values(lb_worker_t *p, jk_logger_t *l)
+void reset_lb_values(lb_worker_t *p, jk_log_context_t *l)
 {
     unsigned int i = 0;
     JK_TRACE_ENTER(l);
@@ -303,7 +303,7 @@ void reset_lb_values(lb_worker_t *p, jk_logger_t *l)
     JK_TRACE_EXIT(l);
 }
 
-static void jk_lb_pull_worker(lb_worker_t *p, int i, jk_logger_t *l)
+static void jk_lb_pull_worker(lb_worker_t *p, int i, jk_log_context_t *l)
 {
     lb_sub_worker_t *w = &p->lb_workers[i];
     if (w->sequence < w->s->h.sequence) {
@@ -316,9 +316,9 @@ static void jk_lb_pull_worker(lb_worker_t *p, int i, jk_logger_t *l)
                    w->name, p->name);
 
         jk_ajp_pull(aw, JK_TRUE, l);
-        strncpy(w->route, w->s->route, JK_SHM_STR_SIZ);
-        strncpy(w->domain, w->s->domain, JK_SHM_STR_SIZ);
-        strncpy(w->redirect, w->s->redirect, JK_SHM_STR_SIZ);
+        jk_shm_str_copy(w->route, w->s->route, l);
+        jk_shm_str_copy(w->domain, w->s->domain, l);
+        jk_shm_str_copy(w->redirect, w->s->redirect, l);
         w->distance = w->s->distance;
         w->activation = w->s->activation;
         w->lb_factor = w->s->lb_factor;
@@ -328,7 +328,7 @@ static void jk_lb_pull_worker(lb_worker_t *p, int i, jk_logger_t *l)
 }
 
 /* Syncing config values from shm */
-void jk_lb_pull(lb_worker_t *p, int locked, jk_logger_t *l)
+void jk_lb_pull(lb_worker_t *p, int locked, jk_log_context_t *l)
 {
     unsigned int i = 0;
 
@@ -367,7 +367,7 @@ void jk_lb_pull(lb_worker_t *p, int locked, jk_logger_t *l)
 }
 
 /* Syncing config values to shm */
-void jk_lb_push(lb_worker_t *p, int locked, int push_all_members, jk_logger_t *l)
+void jk_lb_push(lb_worker_t *p, int locked, int push_all_members, jk_log_context_t *l)
 {
     unsigned int i = 0;
 
@@ -402,9 +402,9 @@ void jk_lb_push(lb_worker_t *p, int locked, int push_all_members, jk_logger_t *l
                        w->name, p->name);
 
             jk_ajp_push(aw, JK_TRUE, l);
-            strncpy(w->s->route, w->route, JK_SHM_STR_SIZ);
-            strncpy(w->s->domain, w->domain, JK_SHM_STR_SIZ);
-            strncpy(w->s->redirect, w->redirect, JK_SHM_STR_SIZ);
+            jk_shm_str_copy(w->s->route, w->route, l);
+            jk_shm_str_copy(w->s->domain, w->domain, l);
+            jk_shm_str_copy(w->s->redirect, w->redirect, l);
             w->s->distance = w->distance;
             w->s->activation = w->activation;
             w->s->lb_factor = w->lb_factor;
@@ -528,7 +528,7 @@ static char *get_cookie(jk_ws_service_t *s, const char *name)
 /* Retrieve session id from the cookie or the parameter
  * (parameter first)
  */
-static char *get_sessionid(jk_ws_service_t *s, lb_worker_t *p, jk_logger_t *l)
+static char *get_sessionid(jk_ws_service_t *s, lb_worker_t *p, jk_log_context_t *l)
 {
     char *val;
     char *session_path;
@@ -568,7 +568,7 @@ static char *get_sessionid(jk_ws_service_t *s, lb_worker_t *p, jk_logger_t *l)
     return val;
 }
 
-static void close_workers(lb_worker_t *p, int num_of_workers, jk_logger_t *l)
+static void close_workers(lb_worker_t *p, int num_of_workers, jk_log_context_t *l)
 {
     int i = 0;
     for (i = 0; i < num_of_workers; i++) {
@@ -589,7 +589,7 @@ static void close_workers(lb_worker_t *p, int num_of_workers, jk_logger_t *l)
 static int recover_workers(lb_worker_t *p,
                            jk_uint64_t curmax,
                            time_t now,
-                           jk_logger_t *l)
+                           jk_log_context_t *l)
 {
     unsigned int i;
     int non_error = 0;
@@ -647,7 +647,7 @@ static int recover_workers(lb_worker_t *p,
 
 static int force_recovery(lb_worker_t *p,
                           int *states,
-                          jk_logger_t *l)
+                          jk_log_context_t *l)
 {
     unsigned int i;
     int forced = 0;
@@ -681,7 +681,7 @@ static int force_recovery(lb_worker_t *p,
  */
 static jk_uint64_t decay_load(lb_worker_t *p,
                               time_t exponent,
-                              jk_logger_t *l)
+                              jk_log_context_t *l)
 {
     unsigned int i;
     jk_uint64_t curmax = 0;
@@ -731,7 +731,7 @@ static jk_uint64_t decay_load(lb_worker_t *p,
     return curmax;
 }
 
-static int JK_METHOD maintain_workers(jk_worker_t *p, time_t now, int global, jk_logger_t *l)
+static int JK_METHOD maintain_workers(jk_worker_t *p, time_t now, int global, jk_log_context_t *l)
 {
     unsigned int i = 0;
     jk_uint64_t curmax = 0;
@@ -793,7 +793,7 @@ static int JK_METHOD maintain_workers(jk_worker_t *p, time_t now, int global, jk
     return JK_TRUE;
 }
 
-static int JK_METHOD shutdown_workers(jk_worker_t *p, jk_logger_t *l)
+static int JK_METHOD shutdown_workers(jk_worker_t *p, jk_log_context_t *l)
 {
     unsigned int i = 0;
 
@@ -819,7 +819,7 @@ static int JK_METHOD shutdown_workers(jk_worker_t *p, jk_logger_t *l)
 static int find_by_session(jk_ws_service_t *s,
                            lb_worker_t *p,
                            const char *session_route,
-                           jk_logger_t *l)
+                           jk_log_context_t *l)
 {
 
     int rc = -1;
@@ -838,7 +838,7 @@ static int find_best_bydomain(jk_ws_service_t *s,
                               lb_worker_t *p,
                               const char *route_or_domain,
                               int *states,
-                              jk_logger_t *l)
+                              jk_log_context_t *l)
 {
     unsigned int i;
     int d = 0;
@@ -888,7 +888,7 @@ static int find_best_bydomain(jk_ws_service_t *s,
 static int find_best_byvalue(jk_ws_service_t *s,
                              lb_worker_t *p,
                              int *states,
-                             jk_logger_t *l)
+                             jk_log_context_t *l)
 {
     unsigned int i;
     unsigned int j;
@@ -935,7 +935,7 @@ static int find_bysession_route(jk_ws_service_t *s,
                                 lb_worker_t *p,
                                 const char *session_route,
                                 int *states,
-                                jk_logger_t *l)
+                                jk_log_context_t *l)
 {
     int uses_domain  = 0;
     int candidate = -1;
@@ -944,7 +944,8 @@ static int find_bysession_route(jk_ws_service_t *s,
     if (candidate < 0) {
         uses_domain = 1;
         candidate = find_best_bydomain(s, p, session_route, states, l);
-    } else {
+    }
+    else {
         s->sticky = JK_TRUE;
     }
     if (candidate >= 0) {
@@ -975,7 +976,8 @@ static int find_bysession_route(jk_ws_service_t *s,
                 candidate = find_best_bydomain(s, p, wr.domain, states, l);
                 if (candidate >= 0) {
                     s->route = wr.domain;
-                } else {
+                }
+                else {
                     s->route = NULL;
                 }
             }
@@ -997,7 +999,7 @@ static int find_bysession_route(jk_ws_service_t *s,
 static int find_failover_worker(jk_ws_service_t *s,
                                 lb_worker_t *p,
                                 int *states,
-                                jk_logger_t *l)
+                                jk_log_context_t *l)
 {
     int rc = -1;
     unsigned int i;
@@ -1019,7 +1021,7 @@ static int find_failover_worker(jk_ws_service_t *s,
 static int find_best_worker(jk_ws_service_t *s,
                             lb_worker_t *p,
                             int *states,
-                            jk_logger_t *l)
+                            jk_log_context_t *l)
 {
     int rc = -1;
 
@@ -1034,7 +1036,7 @@ static int get_most_suitable_worker(jk_ws_service_t *s,
                                     lb_worker_t *p,
                                     char *sessionid,
                                     int *states,
-                                    jk_logger_t *l)
+                                    jk_log_context_t *l)
 {
     int rc = -1;
 
@@ -1150,7 +1152,7 @@ static int get_most_suitable_worker(jk_ws_service_t *s,
 static void lb_add_log_items(jk_ws_service_t *s,
                              const char *const *log_names,
                              lb_sub_worker_t *w,
-                             jk_logger_t *l)
+                             jk_log_context_t *l)
 {
     /* ADJUST JK_LB_NOTES_COUNT WHEN ADDING MORE NOTES HERE! */
     ajp_worker_t *aw = (ajp_worker_t *)w->worker->worker_private;
@@ -1196,7 +1198,7 @@ static void lb_add_log_items(jk_ws_service_t *s,
 
 static int JK_METHOD service(jk_endpoint_t *e,
                              jk_ws_service_t *s,
-                             jk_logger_t *l, int *is_error)
+                             jk_log_context_t *l, int *is_error)
 {
     lb_endpoint_t *p;
     int attempt = 0;
@@ -1281,7 +1283,7 @@ static int JK_METHOD service(jk_endpoint_t *e,
     while (recoverable == JK_TRUE) {
         if (JK_IS_DEBUG_LEVEL(l))
             jk_log(l, JK_LOG_DEBUG, "attempt %d, max attempts %d, worker count %d",
-                    attempt, p->worker->lb_retries, num_of_workers);
+                   attempt, p->worker->lb_retries, num_of_workers);
         if (attempt >= num_of_workers || attempt >= p->worker->lb_retries) {
             retry++;
             if (retry >= p->worker->retries) {
@@ -1388,7 +1390,7 @@ static int JK_METHOD service(jk_endpoint_t *e,
                                                       (s->num_resp_headers + 1) * sizeof(char *));
                     if (!s->resp_headers_names || !s->resp_headers_values) {
                         jk_log(l, JK_LOG_ERROR,
-                            "Failed allocating %d new response headers.", s->num_resp_headers + 1);
+                               "Failed allocating %d new response headers.", s->num_resp_headers + 1);
                         s->resp_headers_names = old_names;
                         s->resp_headers_values = old_values;
                     } else if (s->num_resp_headers) {
@@ -1693,7 +1695,7 @@ static int JK_METHOD service(jk_endpoint_t *e,
     return rc;
 }
 
-static int JK_METHOD done(jk_endpoint_t **e, jk_logger_t *l)
+static int JK_METHOD done(jk_endpoint_t **e, jk_log_context_t *l)
 {
     JK_TRACE_ENTER(l);
 
@@ -1713,7 +1715,7 @@ static int JK_METHOD done(jk_endpoint_t **e, jk_logger_t *l)
 
 static int JK_METHOD validate(jk_worker_t *pThis,
                               jk_map_t *props,
-                              jk_worker_env_t *we, jk_logger_t *l)
+                              jk_worker_env_t *we, jk_log_context_t *l)
 {
     JK_TRACE_ENTER(l);
 
@@ -1743,10 +1745,11 @@ static int JK_METHOD validate(jk_worker_t *pThis,
             }
             memset(p->lb_workers, 0, num_of_workers * sizeof(lb_sub_worker_t));
             for (i = 0; i < num_of_workers; i++) {
-                if (jk_check_attribute_length("host name", worker_names[i], l) == JK_FALSE) {
+                if (jk_shm_str_init(p->lb_workers[i].name, worker_names[i], "name", l) == JK_FALSE) {
                     JK_TRACE_EXIT(l);
                     return JK_FALSE;
                 }
+                jk_shm_str_copy(p->lb_workers[i].route, p->lb_workers[i].name, l);
                 p->lb_workers[i].s = jk_shm_alloc_lb_sub_worker(&p->p, p->s->h.id, worker_names[i], l);
                 if (p->lb_workers[i].s == NULL) {
                     jk_log(l, JK_LOG_ERROR,
@@ -1755,7 +1758,6 @@ static int JK_METHOD validate(jk_worker_t *pThis,
                     return JK_FALSE;
                 }
                 p->lb_workers[i].i = i;
-                strncpy(p->lb_workers[i].name, worker_names[i], JK_SHM_STR_SIZ);
             }
 
             for (i = 0; i < num_of_workers; i++) {
@@ -1794,27 +1796,22 @@ static int JK_METHOD validate(jk_worker_t *pThis,
                 p->lb_workers[i].distance =
                     jk_get_distance(props, worker_names[i]);
                 if ((s = jk_get_worker_route(props, worker_names[i], NULL))) {
-                    if (jk_check_attribute_length("route", s, l) == JK_FALSE) {
+                    if (jk_shm_str_init(p->lb_workers[i].route, s, "route", l) == JK_FALSE) {
                         JK_TRACE_EXIT(l);
                         return JK_FALSE;
                     }
-                    strncpy(p->lb_workers[i].route, s, JK_SHM_STR_SIZ);
                 }
-                else
-                    strncpy(p->lb_workers[i].route, worker_names[i], JK_SHM_STR_SIZ);
                 if ((s = jk_get_worker_domain(props, worker_names[i], NULL))) {
-                    if (jk_check_attribute_length("domain", s, l) == JK_FALSE) {
+                    if (jk_shm_str_init(p->lb_workers[i].domain, s, "domain", l) == JK_FALSE) {
                         JK_TRACE_EXIT(l);
                         return JK_FALSE;
                     }
-                    strncpy(p->lb_workers[i].domain, s, JK_SHM_STR_SIZ);
                 }
                 if ((s = jk_get_worker_redirect(props, worker_names[i], NULL))) {
-                    if (jk_check_attribute_length("redirect", s, l) == JK_FALSE) {
+                    if (jk_shm_str_init(p->lb_workers[i].redirect, s, "redirect", l) == JK_FALSE) {
                         JK_TRACE_EXIT(l);
                         return JK_FALSE;
                     }
-                    strncpy(p->lb_workers[i].redirect, s, JK_SHM_STR_SIZ);
                 }
 
                 p->lb_workers[i].s->lb_value = 0;
@@ -1902,13 +1899,13 @@ static int JK_METHOD validate(jk_worker_t *pThis,
 
 static int JK_METHOD init(jk_worker_t *pThis,
                           jk_map_t *props,
-                          jk_worker_env_t *we, jk_logger_t *log)
+                          jk_worker_env_t *we, jk_log_context_t *l)
 {
     int i;
     const char *s;
 
     lb_worker_t *p = (lb_worker_t *)pThis->worker_private;
-    JK_TRACE_ENTER(log);
+    JK_TRACE_ENTER(l);
 
     p->worker.we = we;
     p->retries = jk_get_worker_retries(props, p->name,
@@ -1935,56 +1932,53 @@ static int JK_METHOD init(jk_worker_t *pThis,
     p->lbmethod = jk_get_lb_method(props, p->name);
 #ifdef JK_ATOMIC_MISSING
     if (p->lbmethod == JK_LB_METHOD_BUSYNESS) {
-        jk_log(log, JK_LOG_WARNING, "Missing support for atomics: "
+        jk_log(l, JK_LOG_WARNING, "Missing support for atomics: "
                "LB method 'busyness' not recommended");
     }
 #endif
     p->lblock   = jk_get_lb_lock(props, p->name);
     s = jk_get_lb_session_cookie(props, p->name, JK_SESSION_IDENTIFIER);
-    if (jk_check_attribute_length("session_cookie", s, log) == JK_FALSE) {
-        JK_TRACE_EXIT(log);
+    if (jk_shm_str_init(p->session_cookie, s, "session_cookie", l) == JK_FALSE) {
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
     }
-    strncpy(p->session_cookie, s, JK_SHM_STR_SIZ);
     s = jk_get_lb_session_path(props, p->name, JK_PATH_SESSION_IDENTIFIER);
-    if (jk_check_attribute_length("session_path", s, log) == JK_FALSE) {
-        JK_TRACE_EXIT(log);
+    if (jk_shm_str_init(p->session_path, s, "session_path", l) == JK_FALSE) {
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
     }
-    strncpy(p->session_path, s, JK_SHM_STR_SIZ);
     p->set_session_cookie = jk_get_lb_set_session_cookie(props, p->name, JK_FALSE);
     s = jk_get_lb_session_cookie_path(props, p->name, "/");
-    if (jk_check_attribute_length("session_cookie_path", s, log) == JK_FALSE) {
-        JK_TRACE_EXIT(log);
+    if (jk_shm_str_init(p->session_cookie_path, s, "session_cookie_path", l) == JK_FALSE) {
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
     }
-    strncpy(p->session_cookie_path, s, JK_SHM_STR_SIZ);
 
     JK_INIT_CS(&(p->cs), i);
     if (i == JK_FALSE) {
-        jk_log(log, JK_LOG_ERROR,
+        jk_log(l, JK_LOG_ERROR,
                "creating thread lock (errno=%d)",
                errno);
-        JK_TRACE_EXIT(log);
+        JK_TRACE_EXIT(l);
         return JK_FALSE;
     }
     if (p->s->h.sequence == 0) {
         /* Set configuration data to shared memory
          */
-        jk_lb_push(p, JK_TRUE, JK_FALSE, log);
+        jk_lb_push(p, JK_TRUE, JK_FALSE, l);
     }
     else {
         /* Shared memory for this worker is already configured.
          * Update with runtime data
          */
-        jk_lb_pull(p, JK_TRUE, log);
+        jk_lb_pull(p, JK_TRUE, l);
     }
-    JK_TRACE_EXIT(log);
+    JK_TRACE_EXIT(l);
     return JK_TRUE;
 }
 
 static int JK_METHOD get_endpoint(jk_worker_t *pThis,
-                                  jk_endpoint_t **pend, jk_logger_t *l)
+                                  jk_endpoint_t **pend, jk_log_context_t *l)
 {
     JK_TRACE_ENTER(l);
 
@@ -2014,7 +2008,7 @@ static int JK_METHOD get_endpoint(jk_worker_t *pThis,
     return JK_FALSE;
 }
 
-static int JK_METHOD destroy(jk_worker_t **pThis, jk_logger_t *l)
+static int JK_METHOD destroy(jk_worker_t **pThis, jk_log_context_t *l)
 {
     JK_TRACE_ENTER(l);
 
@@ -2036,7 +2030,7 @@ static int JK_METHOD destroy(jk_worker_t **pThis, jk_logger_t *l)
 }
 
 int JK_METHOD lb_worker_factory(jk_worker_t **w,
-                                const char *name, jk_logger_t *l)
+                                const char *name, jk_log_context_t *l)
 {
     JK_TRACE_ENTER(l);
 
@@ -2055,7 +2049,10 @@ int JK_METHOD lb_worker_factory(jk_worker_t **w,
             JK_TRACE_EXIT(l);
             return 0;
         }
-        strncpy(private_data->name, name, JK_SHM_STR_SIZ);
+        if (jk_shm_str_init(private_data->name, name, "name", l) == JK_FALSE) {
+            JK_TRACE_EXIT(l);
+            return 0;
+        }
         private_data->lb_workers = NULL;
         private_data->num_of_workers = 0;
         private_data->worker.worker_private = private_data;
