@@ -19,7 +19,6 @@
  * Description: Shared Memory support                                      *
  * Author:      Mladen Turk <mturk@jboss.com>                              *
  * Author:      Rainer Jung <rjung@apache.org>                             *
- * Version:     $Revision: 1838573 $                                        *
  ***************************************************************************/
 
 #include "jk_global.h"
@@ -119,7 +118,7 @@ static size_t jk_shm_calculate_slot_size()
 }
 
 /* Calculate needed shm size */
-int jk_shm_calculate_size(jk_map_t *init_data, jk_logger_t *l)
+int jk_shm_calculate_size(jk_map_t *init_data, jk_log_context_t *l)
 {
     char **worker_list;
     size_t needed_slot_size = 0;
@@ -191,7 +190,7 @@ int jk_shm_calculate_size(jk_map_t *init_data, jk_logger_t *l)
 #if defined (WIN32)
 
 /* Use plain memory */
-int jk_shm_open(const char *fname, int sz, jk_logger_t *l)
+int jk_shm_open(const char *fname, int sz, jk_log_context_t *l)
 {
     int rc = -1;
     int attached = 0;
@@ -370,7 +369,7 @@ int jk_shm_open(const char *fname, int sz, jk_logger_t *l)
     return 0;
 }
 
-int jk_shm_attach(const char *fname, int sz, jk_logger_t *l)
+int jk_shm_attach(const char *fname, int sz, jk_log_context_t *l)
 {
     JK_TRACE_ENTER(l);
     if (!jk_shm_open(fname, sz, l)) {
@@ -378,10 +377,10 @@ int jk_shm_attach(const char *fname, int sz, jk_logger_t *l)
             jk_shmem.attached = 1;
             if (JK_IS_DEBUG_LEVEL(l)) {
                 jk_log(l, JK_LOG_DEBUG,
-                   "Attached shared memory %s [%d] size=%u free=%u addr=%#lx",
-                   jk_shm_name(), jk_shmem.hdr->h.data.childs, jk_shmem.size,
-                   jk_shmem.hdr->h.data.size - jk_shmem.hdr->h.data.pos,
-                   jk_shmem.hdr);
+                       "Attached shared memory %s [%d] size=%u free=%u addr=%#lx",
+                       jk_shm_name(), jk_shmem.hdr->h.data.childs, jk_shmem.size,
+                       jk_shmem.hdr->h.data.size - jk_shmem.hdr->h.data.pos,
+                       jk_shmem.hdr);
             }
         }
         JK_TRACE_EXIT(l);
@@ -393,7 +392,7 @@ int jk_shm_attach(const char *fname, int sz, jk_logger_t *l)
     }
 }
 
-void jk_shm_close(jk_logger_t *l)
+void jk_shm_close(jk_log_context_t *l)
 {
     if (jk_shm_inited_cs) {
         JK_ENTER_CS(&jk_shmem.cs);
@@ -448,7 +447,7 @@ void jk_shm_close(jk_logger_t *l)
 #define MAP_FILE    (0)
 #endif
 
-static int do_shm_open_lock(const char *fname, int attached, jk_logger_t *l)
+static int do_shm_open_lock(const char *fname, int attached, jk_log_context_t *l)
 {
     int rc;
     char flkname[256];
@@ -536,7 +535,7 @@ static int do_shm_open_lock(const char *fname, int attached, jk_logger_t *l)
 }
 
 static int do_shm_open(const char *fname, int attached,
-                       int sz, jk_logger_t *l)
+                       int sz, jk_log_context_t *l)
 {
     int rc;
     int fd;
@@ -724,17 +723,17 @@ static int do_shm_open(const char *fname, int attached,
     return 0;
 }
 
-int jk_shm_open(const char *fname, int sz, jk_logger_t *l)
+int jk_shm_open(const char *fname, int sz, jk_log_context_t *l)
 {
     return do_shm_open(fname, 0, sz, l);
 }
 
-int jk_shm_attach(const char *fname, int sz, jk_logger_t *l)
+int jk_shm_attach(const char *fname, int sz, jk_log_context_t *l)
 {
     return do_shm_open(fname, 1, sz, l);
 }
 
-void jk_shm_close(jk_logger_t *l)
+void jk_shm_close(jk_log_context_t *l)
 {
 #ifdef AS400_UTF8
     char *wptr;
@@ -807,14 +806,60 @@ void jk_shm_close(jk_logger_t *l)
 
 #endif
 
+static int jk_shm_check_str_length(const char *name, const char *value,
+                                   jk_log_context_t *l)
+{
+    size_t len = strlen(value);
+    if (len >= JK_SHM_STR_SIZ) {
+        jk_log(l, JK_LOG_ERROR,
+               "Worker %s '%s' is %d bytes too long, "
+               "a maximum of %d bytes is supported",
+               name, value, len - (JK_SHM_STR_SIZ - 1), JK_SHM_STR_SIZ - 1);
+        return JK_FALSE;
+    }
+    return JK_TRUE;
+}
+
+int jk_shm_str_init(shm_str dst, const char *src,
+                    const char *name, jk_log_context_t *l)
+{
+    if (jk_shm_check_str_length(name, src, l) == JK_FALSE) {
+        return JK_FALSE;
+    }
+    strncpy(dst, src, JK_SHM_STR_SIZ - 1);
+    dst[JK_SHM_STR_SIZ - 1] = '\0';
+    return JK_TRUE;
+}
+
+int jk_shm_str_init_ne(shm_str dst, const char *src,
+                       const char *name, jk_log_context_t *l)
+{
+    if (jk_shm_check_str_length(name, src, l) == JK_FALSE) {
+        return -1;
+    }
+    if (!strncmp(dst, src, JK_SHM_STR_SIZ)) {
+        return 0;
+    }
+    strncpy(dst, src, JK_SHM_STR_SIZ - 1);
+    dst[JK_SHM_STR_SIZ - 1] = '\0';
+    return 1;
+}
+
+void jk_shm_str_copy(shm_str dst, shm_str src,
+                     jk_log_context_t *l)
+{
+    memcpy((dst), (src), JK_SHM_STR_SIZ);
+}
+
 jk_shm_worker_header_t *jk_shm_alloc_worker(jk_pool_t *p, int type,
                                             int parent_id, const char *name,
-                                            jk_logger_t *l)
+                                            jk_log_context_t *l)
 {
     unsigned int i;
     jk_shm_worker_header_t *w = 0;
+    shm_str str;
 
-    if (jk_check_attribute_length("name", name, l) == JK_FALSE) {
+    if (jk_shm_str_init(str, name, "name", l) == JK_FALSE) {
         return NULL;
     }
 
@@ -833,7 +878,7 @@ jk_shm_worker_header_t *jk_shm_alloc_worker(jk_pool_t *p, int type,
         if ((jk_shmem.hdr->h.data.size - jk_shmem.hdr->h.data.pos) >= JK_SHM_SLOT_SIZE) {
             w = (jk_shm_worker_header_t *)(jk_shmem.hdr->buf + jk_shmem.hdr->h.data.pos);
             memset(w, 0, JK_SHM_SLOT_SIZE);
-            strncpy(w->name, name, JK_SHM_STR_SIZ);
+            jk_shm_str_copy(w->name, str, l);
             jk_shmem.hdr->h.data.workers++;
             w->id = jk_shmem.hdr->h.data.workers;
             w->type = type;
@@ -854,7 +899,7 @@ jk_shm_worker_header_t *jk_shm_alloc_worker(jk_pool_t *p, int type,
         w = (jk_shm_worker_header_t *)jk_pool_alloc(p, JK_SHM_SLOT_SIZE);
         if (w) {
             memset(w, 0, JK_SHM_SLOT_SIZE);
-            strncpy(w->name, name, JK_SHM_STR_SIZ);
+            jk_shm_str_copy(w->name, str, l);
             w->id = 0;
             w->type = type;
             w->parent_id = parent_id;
@@ -926,21 +971,21 @@ int jk_shm_unlock()
 }
 
 jk_shm_ajp_worker_t *jk_shm_alloc_ajp_worker(jk_pool_t *p, const char *name,
-                                             jk_logger_t *l)
+                                             jk_log_context_t *l)
 {
     return (jk_shm_ajp_worker_t *)jk_shm_alloc_worker(p,
                                     JK_AJP13_WORKER_TYPE, 0, name, l);
 }
 
 jk_shm_lb_sub_worker_t *jk_shm_alloc_lb_sub_worker(jk_pool_t *p, int lb_id, const char *name,
-                                                   jk_logger_t *l)
+                                                   jk_log_context_t *l)
 {
     return (jk_shm_lb_sub_worker_t *)jk_shm_alloc_worker(p,
                                     JK_LB_SUB_WORKER_TYPE, lb_id, name, l);
 }
 
 jk_shm_lb_worker_t *jk_shm_alloc_lb_worker(jk_pool_t *p, const char *name,
-                                           jk_logger_t *l)
+                                           jk_log_context_t *l)
 {
     return (jk_shm_lb_worker_t *)jk_shm_alloc_worker(p,
                                     JK_LB_WORKER_TYPE, 0, name, l);
